@@ -1,8 +1,9 @@
 #!/usr/bin/env python2
+import argparse
+import sys
+
+from pwn import *
 
-import argparse, sys
-from pwnlib.elf import ELF
-from pwnlib.util.fiddling import unhex
 p = argparse.ArgumentParser()
 p.add_argument('elf',help="File to patch")
 p.add_argument('offset',help="Offset to patch in virtual address (hex encoded)")
@@ -17,7 +18,9 @@ def main():
 
     offset = int(a.offset, 16)
     bytes  = unhex(a.bytes)
-    elf    = ELF(a.elf)
+
+    with context.silent:
+        elf    = ELF(a.elf)
 
     elf.write(offset, bytes)
     sys.stdout.write(elf.get_data())
