@@ -1,5 +1,6 @@
 #!/usr/bin/env python2
-import sys, argparse
+import argparse
+import sys
 
 parser = argparse.ArgumentParser(description='''
 Hex-encodes data provided on the command line or via stdin.
@@ -14,4 +15,4 @@ def main():
     else:
         print ' '.join(args.data).encode('hex')
 
-if __name__ == '__main__': main()
\ No newline at end of file
+if __name__ == '__main__': main()
