@@ -1,5 +1,6 @@
 #!/usr/bin/env python2
-import sys, argparse
+import argparse
+import sys
 from string import whitespace
 
 parser = argparse.ArgumentParser(description='''
@@ -19,4 +20,4 @@ def main():
     except TypeError, e:
         sys.stderr.write(str(e) + '\n')
 
-if __name__ == '__main__': main()
\ No newline at end of file
+if __name__ == '__main__': main()
