@@ -1,9 +1,12 @@
-from .sock     import sock
-from ..timeout import Timeout
-from ..context import context
-import socket, errno, logging
+import errno
+import socket
 
-log = logging.getLogger(__name__)
+from ..context import context
+from ..log import getLogger
+from ..timeout import Timeout
+from .sock import sock
+
+log = getLogger(__name__)
 
 class listen(sock):
     """Creates an TCP or UDP-socket to receive data on. It supports
@@ -12,7 +15,7 @@ class listen(sock):
     The returned object supports all the methods from
     :class:`pwnlib.tubes.sock` and :class:`pwnlib.tubes.tube`.
 
-    Args:
+    Arguments:
         port(int): The port to connect to.
         bindaddr(str): The address to bind to.
         fam: The string "any", "ipv4" or "ipv6" or an integer to pass to :func:`socket.getaddrinfo`.
@@ -22,21 +25,25 @@ class listen(sock):
 
     def __init__(self, port=0, bindaddr = "0.0.0.0",
                  fam = "any", typ = "tcp",
-                 timeout = Timeout.default):
-        super(listen, self).__init__(timeout)
+                 timeout = Timeout.default, level = None):
+        super(listen, self).__init__(timeout, level = level)
 
         port = int(port)
+        fam  = {socket.AF_INET: 'ipv4',
+                socket.AF_INET6: 'ipv6'}.get(fam, fam)
 
         if fam == 'any':
             fam = socket.AF_UNSPEC
-        elif fam == 4 or fam.lower() in ['ipv4', 'ip4', 'v4', '4']:
+        elif fam.lower() in ['ipv4', 'ip4', 'v4', '4']:
             fam = socket.AF_INET
-        elif fam == 6 or fam.lower() in ['ipv6', 'ip6', 'v6', '6']:
+        elif fam.lower() in ['ipv6', 'ip6', 'v6', '6']:
             fam = socket.AF_INET6
+            if bindaddr == '0.0.0.0':
+                bindaddr = '::'
         elif isinstance(fam, (int, long)):
             pass
         else:
-            log.error("remote(): family %r is not supported" % fam)
+            self.error("remote(): family %r is not supported" % fam)
 
         if typ == "tcp":
             typ = socket.SOCK_STREAM
@@ -45,9 +52,9 @@ class listen(sock):
         elif isinstance(typ, (int, long)):
             pass
         else:
-            log.error("remote(): type %r is not supported" % typ)
+            self.error("remote(): type %r is not supported" % typ)
 
-        h = log.waitfor('Trying to bind to %s on port %d' % (bindaddr, port))
+        h = self.waitfor('Trying to bind to %s on port %d' % (bindaddr, port))
 
         for res in socket.getaddrinfo(bindaddr, port, fam, typ, 0, socket.AI_PASSIVE):
             self.family, self.type, self.proto, self.canonname, self.sockaddr = res
@@ -65,11 +72,11 @@ class listen(sock):
             break
         else:
             h.failure()
-            log.error("Could not bind to %s on port %d" % (bindaddr, port))
+            self.error("Could not bind to %s on port %d" % (bindaddr, port))
 
         h.success()
 
-        h = log.waitfor('Waiting for connections on %s:%s' % (self.lhost, self.lport))
+        h = self.waitfor('Waiting for connections on %s:%s' % (self.lhost, self.lport))
 
         def accepter():
             while True:
@@ -78,16 +85,17 @@ class listen(sock):
                         self.sock, rhost = listen_sock.accept()
                         listen_sock.close()
                     else:
-                        self.buffer, rhost = listen_sock.recvfrom(4096)
+                        data, rhost = listen_sock.recvfrom(4096)
                         listen_sock.connect(rhost)
                         self.sock = listen_sock
+                        self.unrecv(data)
                     self.settimeout(self.timeout)
                     break
                 except socket.error as e:
                     if e.errno == errno.EINTR:
                         continue
                     h.failure()
-                    log.exception("Socket failure while waiting for connection")
+                    self.exception("Socket failure while waiting for connection")
                     self.sock = None
                     return
 
@@ -101,6 +109,7 @@ class listen(sock):
     def spawn_process(self, *args, **kwargs):
         def accepter():
             self.wait_for_connection()
+            self.sock.setblocking(1)
             p = super(listen, self).spawn_process(*args, **kwargs)
             p.wait()
             self.close()
@@ -115,8 +124,7 @@ class listen(sock):
 
     def __getattr__(self, key):
         if key == 'sock':
-            while self._accepter.is_alive():
-                self._accepter.join(timeout = 0.1)
+            self._accepter.join(timeout = self.timeout)
             if 'sock' in self.__dict__:
                 return self.sock
             else:
