@@ -1,9 +1,11 @@
-from .sock import sock
-from ..timeout import Timeout
-import socket, logging
+import socket
 import ssl as _ssl
 
-log = logging.getLogger(__name__)
+from ..log import getLogger
+from ..timeout import Timeout
+from .sock import sock
+
+log = getLogger(__name__)
 
 class remote(sock):
     r"""Creates a TCP or UDP-connection to a remote host. It supports
@@ -12,7 +14,7 @@ class remote(sock):
     The returned object supports all the methods from
     :class:`pwnlib.tubes.sock` and :class:`pwnlib.tubes.tube`.
 
-    Args:
+    Arguments:
         host(str): The host to connect to.
         port(int): The port to connect to.
         fam: The string "any", "ipv4" or "ipv6" or an integer to pass to :func:`socket.getaddrinfo`.
@@ -43,8 +45,8 @@ class remote(sock):
 
     def __init__(self, host, port,
                  fam = "any", typ = "tcp",
-                 timeout = Timeout.default, ssl=False, sock=None):
-        super(remote, self).__init__(timeout)
+                 timeout = Timeout.default, ssl=False, sock=None, level = None):
+        super(remote, self).__init__(timeout, level = level)
 
         self.rport  = int(port)
         self.rhost  = host
@@ -58,8 +60,12 @@ class remote(sock):
         else:
             typ = self._get_type(typ)
             fam = self._get_family(fam)
-            self.sock   = self._connect(fam, typ)
-
+            try:
+                self.sock   = self._connect(fam, typ)
+            except socket.gaierror as e:
+                if e.errno != socket.EAI_NONAME:
+                    raise
+                log.error('Could not resolve hostname: %r' % host)
         if self.sock:
             self.settimeout(self.timeout)
             self.lhost, self.lport = self.sock.getsockname()[:2]
@@ -80,7 +86,7 @@ class remote(sock):
         elif fam.lower() in ['ipv6', 'ip6', 'v6', '6']:
             fam = socket.AF_INET6
         else:
-            log.error("remote(): family %r is not supported" % fam)
+            self.error("remote(): family %r is not supported" % fam)
 
         return fam
 
@@ -94,7 +100,7 @@ class remote(sock):
         elif typ == "udp":
             typ = socket.SOCK_DGRAM
         else:
-            log.error("remote(): type %r is not supported" % typ)
+            self.error("remote(): type %r is not supported" % typ)
 
         return typ
 
@@ -102,7 +108,7 @@ class remote(sock):
         sock    = None
         timeout = self.timeout
 
-        h = log.waitfor('Opening connection to %s on port %d' % (self.rhost, self.rport))
+        h = self.waitfor('Opening connection to %s on port %d' % (self.rhost, self.rport))
 
         for res in socket.getaddrinfo(self.rhost, self.rport, fam, typ, 0, socket.AI_PASSIVE):
             self.family, self.type, self.proto, _canonname, sockaddr = res
@@ -127,7 +133,7 @@ class remote(sock):
                 pass
         else:
             h.failure()
-            log.error("Could not connect to %s on port %d" % (self.rhost, self.rport))
+            self.error("Could not connect to %s on port %d" % (self.rhost, self.rport))
 
         h.success()
         return sock
@@ -149,3 +155,15 @@ class remote(sock):
         s = socket
         host, port = s.getpeername()
         return remote(host, port, fam=s.family, typ=s.type, sock=s)
+
+class tcp(remote):
+    def __init__(self, host, port,
+                 fam = "any", typ = "tcp",
+                 timeout = Timeout.default, ssl=False, sock=None, level = None):
+        return super(tcp, self).__init__(host, port, fam, typ, timeout, ssl, sock, level)
+
+class udp(remote):
+    def __init__(self, host, port,
+                 fam = "any", typ = "udp",
+                 timeout = Timeout.default, ssl=False, sock=None, level = None):
+        return super(udp, self).__init__(host, port, fam, typ, timeout, ssl, sock, level)
