@@ -1,20 +1,35 @@
 # -*- coding: utf-8 -*-
-import re, base64, random, string
-from . import packing, lists
-from .cyclic import cyclic_find
+import base64
+import random
+import re
+import string
+import StringIO
+
+from . import lists
+from . import packing
 from ..context import context
-from ..term    import text
+from ..log import getLogger
+from ..term import text
+from .cyclic import cyclic
+from .cyclic import cyclic_find
+
+log = getLogger(__name__)
 
 def unhex(s):
-    """unhex(s) -> str
+    r"""unhex(s) -> str
 
     Hex-decodes a string.
 
     Example:
 
-      >>> unhex("74657374")
-      'test'
-"""
+        >>> unhex("74657374")
+        'test'
+        >>> unhex("F\n")
+        '\x0f'
+    """
+    s = s.strip()
+    if len(s) % 2 != 0:
+        s = '0' + s
     return s.decode('hex')
 
 def enhex(x):
@@ -24,9 +39,9 @@ def enhex(x):
 
     Example:
 
-      >>> enhex("test")
-      '74657374'
-"""
+        >>> enhex("test")
+        '74657374'
+    """
     return x.encode('hex')
 
 def urlencode(s):
@@ -36,9 +51,9 @@ def urlencode(s):
 
     Example:
 
-      >>> urlencode("test")
-      '%74%65%73%74'
-"""
+        >>> urlencode("test")
+        '%74%65%73%74'
+    """
     return ''.join(['%%%02x' % ord(c) for c in s])
 
 def urldecode(s, ignore_invalid = False):
@@ -48,15 +63,15 @@ def urldecode(s, ignore_invalid = False):
 
     Example:
 
-      >>> urldecode("test%20%41")
-      'test A'
-      >>> urldecode("%qq")
-      Traceback (most recent call last):
-          ...
-      ValueError: Invalid input to urldecode
-      >>> urldecode("%qq", ignore_invalid = True)
-      '%qq'
-"""
+        >>> urldecode("test%20%41")
+        'test A'
+        >>> urldecode("%qq")
+        Traceback (most recent call last):
+        ...
+        ValueError: Invalid input to urldecode
+        >>> urldecode("%qq", ignore_invalid = True)
+        '%qq'
+    """
     res = ''
     n = 0
     while n < len(s):
@@ -80,23 +95,26 @@ def bits(s, endian = 'big', zero = 0, one = 1):
 
     Converts the argument a list of bits.
 
-    Args:
-      s: A string or number to be converted into bits.
-      endian (str): The binary endian, default 'big'.
-      zero: The representing a 0-bit.
-      one: The representing a 1-bit.
+    Arguments:
+        s: A string or number to be converted into bits.
+        endian (str): The binary endian, default 'big'.
+        zero: The representing a 0-bit.
+        one: The representing a 1-bit.
 
     Returns:
-      A list consisting of the values specified in `zero` and `one`.
+        A list consisting of the values specified in `zero` and `one`.
 
     Examples:
 
-      >>> bits(511, zero = "+", one = "-")
-      ['+', '+', '+', '+', '+', '+', '+', '-', '-', '-', '-', '-', '-', '-', '-', '-']
-      >>> sum(bits("test"))
-      17
-"""
-
+        >>> bits(511, zero = "+", one = "-")
+        ['+', '+', '+', '+', '+', '+', '+', '-', '-', '-', '-', '-', '-', '-', '-', '-']
+        >>> sum(bits("test"))
+        17
+        >>> bits(0)
+        [0, 0, 0, 0, 0, 0, 0, 0]
+    """
+    if s < 0:
+        s = s & ((1<<context.bits)-1)
 
     if endian not in ['little', 'big']:
         raise ValueError("bits(): 'endian' must be either 'little' or 'big'")
@@ -116,6 +134,8 @@ def bits(s, endian = 'big', zero = 0, one = 1):
             else:
                 out += byte[::-1]
     elif isinstance(s, (int, long)):
+        if s == 0:
+            out.append(zero)
         while s:
             bit, s = one if s & 1 else zero, s >> 1
             out.append(bit)
@@ -139,7 +159,7 @@ def bits_str(s, endian = 'big', zero = '0', one = '1'):
        '0000000111111111'
        >>> bits_str("bits_str", endian = "little")
        '0100011010010110001011101100111011111010110011100010111001001110'
-"""
+    """
     return ''.join(bits(s, endian, zero, one))
 
 def unbits(s, endian = 'big'):
@@ -147,7 +167,7 @@ def unbits(s, endian = 'big'):
 
     Converts an iterable of bits into a string.
 
-    Args:
+    Arguments:
        s: Iterable of bits
        endian (str):  The string "little" or "big", which specifies the bits endianness.
 
@@ -195,9 +215,9 @@ def bitswap(s):
     Reverses the bits in every byte of a given string.
 
     Example:
-      >>> bitswap("1234")
-      '\\x8cL\\xcc,'
-"""
+        >>> bitswap("1234")
+        '\\x8cL\\xcc,'
+    """
 
     out = []
 
@@ -211,20 +231,20 @@ def bitswap_int(n, width):
 
     Reverses the bits of a numbers and returns the result as a new number.
 
-    Args:
-      n (int): The number to swap.
-      width (int): The width of the integer
+    Arguments:
+        n (int): The number to swap.
+        width (int): The width of the integer
 
     Examples:
-      >>> hex(bitswap_int(0x1234, 8))
-      '0x2c'
-      >>> hex(bitswap_int(0x1234, 16))
-      '0x2c48'
-      >>> hex(bitswap_int(0x1234, 24))
-      '0x2c4800'
-      >>> hex(bitswap_int(0x1234, 25))
-      '0x589000'
-"""
+        >>> hex(bitswap_int(0x1234, 8))
+        '0x2c'
+        >>> hex(bitswap_int(0x1234, 16))
+        '0x2c48'
+        >>> hex(bitswap_int(0x1234, 24))
+        '0x2c4800'
+        >>> hex(bitswap_int(0x1234, 25))
+        '0x589000'
+    """
     # Make n fit inside the width
     n &= (1 << width) - 1
 
@@ -267,7 +287,7 @@ def xor(*args, **kwargs):
     then xors them together. If the end of a string is reached, it wraps
     around in the string.
 
-    Args:
+    Arguments:
        args: The arguments to be xor'ed together.
        cut: How long a string should be returned.
             Can be either 'min'/'max'/'left'/'right' or a number.
@@ -278,7 +298,7 @@ def xor(*args, **kwargs):
     Example:
        >>> xor('lol', 'hello', 42)
        '. ***'
-"""
+    """
 
     cut = kwargs.pop('cut', 'max')
 
@@ -318,25 +338,30 @@ def xor_pair(data, avoid = '\x00\n'):
     Finds two strings that will xor into a given string, while only
     using a given alphabet.
 
-    Args:
-      data (str): The desired string.
-      avoid: The list of disallowed characters. Defaults to nulls and newlines.
+    Arguments:
+        data (str): The desired string.
+        avoid: The list of disallowed characters. Defaults to nulls and newlines.
 
     Returns:
-      Two strings which will xor to the given string. If no such two strings exist, then None is returned.
+        Two strings which will xor to the given string. If no such two strings exist, then None is returned.
 
     Example:
 
-      >>> xor_pair("test")
-      ('\\x01\\x01\\x01\\x01', 'udru')
-"""
+        >>> xor_pair("test")
+        ('\\x01\\x01\\x01\\x01', 'udru')
+    """
 
-    alphabet = ''.join(chr(n) for n in range(256) if chr(n) not in avoid)
+    if isinstance(data, (int, long)):
+        data = packing.pack(data)
+
+    alphabet = list(chr(n) for n in range(256) if chr(n) not in avoid)
 
     res1 = ''
     res2 = ''
 
     for c1 in data:
+        if context.randomize:
+            random.shuffle(alphabet)
         for c2 in alphabet:
             c3 = chr(ord(c1) ^ ord(c2))
             if c3 in alphabet:
@@ -348,24 +373,71 @@ def xor_pair(data, avoid = '\x00\n'):
 
     return res1, res2
 
+def xor_key(data, avoid='\x00\n', size=None):
+    r"""xor_key(data, size=None, avoid='\x00\n') -> None or (int, str)
+
+    Finds a ``size``-width value that can be XORed with a string
+    to produce ``data``, while neither the XOR value or XOR string
+    contain any bytes in ``avoid``.
+
+    Arguments:
+        data (str): The desired string.
+        avoid: The list of disallowed characters. Defaults to nulls and newlines.
+        size (int): Size of the desired output value, default is word size.
+
+    Returns:
+        A tuple containing two strings; the XOR key and the XOR string.
+        If no such pair exists, None is returned.
+
+    Example:
+
+        >>> xor_key("Hello, world")
+        ('\x01\x01\x01\x01', 'Idmmn-!vnsme')
+    """
+    size = size or context.bytes
+
+    if len(data) % size:
+        log.error("Data must be padded to size for xor_key")
+
+    words    = lists.group(size, data)
+    columns  = [''] * size
+    for word in words:
+        for i,byte in enumerate(word):
+            columns[i] += byte
+
+    alphabet = list(chr(n) for n in range(256) if chr(n) not in avoid)
+
+    result = ''
+
+    for column in columns:
+        if context.randomize:
+            random.shuffle(alphabet)
+        for c2 in alphabet:
+            if all(chr(ord(c)^ord(c2)) in alphabet for c in column):
+                result += c2
+                break
+        else:
+            return None
+
+    return result, xor(data, result)
 
 def randoms(count, alphabet = string.lowercase):
     """randoms(count, alphabet = string.lowercase) -> str
 
     Returns a random string of a given length using only the specified alphabet.
 
-    Args:
-      count (int): The length of the desired string.
-      alphabet: The alphabet of allowed characters. Defaults to all lowercase characters.
+    Arguments:
+        count (int): The length of the desired string.
+        alphabet: The alphabet of allowed characters. Defaults to all lowercase characters.
 
     Returns:
-      A random string.
+        A random string.
 
     Example:
 
-      >>> randoms(10) #doctest: +SKIP
-      'evafjilupm'
-"""
+        >>> randoms(10) #doctest: +SKIP
+        'evafjilupm'
+    """
 
     return ''.join(random.choice(alphabet) for _ in xrange(count))
 
@@ -377,22 +449,22 @@ def rol(n, k, word_size = None):
 
     When `n` is a list, tuple or string, this is ``n[k % len(n):] + n[:k % len(n)]``.
 
-    Args:
-      n: The value to rotate.
-      k(int): The rotation amount. Can be a positive or negative number.
-      word_size(int): If `n` is a number, then this is the assumed bitsize of `n`.  Defaults to :data:`pwnlib.context.word_size` if `None` .
+    Arguments:
+        n: The value to rotate.
+        k(int): The rotation amount. Can be a positive or negative number.
+        word_size(int): If `n` is a number, then this is the assumed bitsize of `n`.  Defaults to :data:`pwnlib.context.word_size` if `None` .
 
     Example:
 
-      >>> rol('abcdefg', 2)
-      'cdefgab'
-      >>> rol('abcdefg', -2)
-      'fgabcde'
-      >>> hex(rol(0x86, 3, 8))
-      '0x34'
-      >>> hex(rol(0x86, -3, 8))
-      '0xd0'
-"""
+        >>> rol('abcdefg', 2)
+        'cdefgab'
+        >>> rol('abcdefg', -2)
+        'fgabcde'
+        >>> hex(rol(0x86, 3, 8))
+        '0x34'
+        >>> hex(rol(0x86, -3, 8))
+        '0xd0'
+    """
 
     word_size = word_size or context.word_size
 
@@ -416,7 +488,33 @@ def rol(n, k, word_size = None):
 def ror(n, k, word_size = None):
     """A simple wrapper around :func:`rol`, which negates the values of `k`."""
 
-    return ror(n, -k, word_size)
+    return rol(n, -k, word_size)
+
+def naf(n):
+    """naf(int) -> int generator
+
+    Returns a generator for the non-adjacent form (NAF[1]) of a number, `n`.  If
+    `naf(n)` generates `z_0, z_1, ...`, then `n == z_0 + z_1 * 2 + z_2 * 2**2,
+    ...`.
+
+    [1] https://en.wikipedia.org/wiki/Non-adjacent_form
+
+    Example:
+
+      >>> n = 45
+      >>> m = 0
+      >>> x = 1
+      >>> for z in naf(n):
+      ...     m += x * z
+      ...     x *= 2
+      >>> n == m
+      True
+
+    """
+    while n:
+        z = 2 - n % 4 if n & 1 else 0
+        n = (n - z) // 2
+        yield z
 
 def isprint(c):
     """isprint(c) -> bool
@@ -430,14 +528,14 @@ def hexii(s, width = 16, skip = True):
 
     Return a HEXII-dump of a string.
 
-    Args:
-      s(str): The string to dump
-      width(int): The number of characters per line
-      skip(bool): Should repeated lines be replaced by a "*"
+    Arguments:
+        s(str): The string to dump
+        width(int): The number of characters per line
+        skip(bool): Should repeated lines be replaced by a "*"
 
     Returns:
-      A HEXII-dump in the form of a string.
-"""
+        A HEXII-dump in the form of a string.
+    """
 
     return hexdump(s, width, skip, True)
 
@@ -456,48 +554,40 @@ default_style = {
     'marker':       text.gray if text.has_gray else text.blue,
     'nonprintable': text.gray if text.has_gray else text.blue,
     '00':           text.red,
+    '0a':           text.red,
     'ff':           text.green,
 }
 
+cyclic_pregen = ''
+
 def sequential_lines(a,b):
-    if len(a) != len(b) or len(a) < 4:
-        return False
+    return (a+b) in cyclic_pregen
 
-    all_chars = sorted(set(a+b))
+def update_cyclic_pregenerated(size):
+    global cyclic_pregen
+    cyclic_pregen = cyclic(size)
 
-    alphabet = ''
-    if all(a in string.lowercase for a in all_chars):
-        alphabet = string.lowercase
-    if all(a in string.uppercase for a in all_chars):
-        alphabet = string.uppercase
-
-    # Check each set of four
-    for i in range(0, len(a)-3):
-        A = cyclic_find(a[i:i+4], alphabet)
-        B = cyclic_find(b[i:i+4], alphabet)
-        if A+len(a) != B:
-            return False
-    return True
-
-def hexdump_iter(s, width = 16, skip = True, hexii = False, begin = 0,
-                 style = None, highlight = None):
+def hexdump_iter(fd, width=16, skip=True, hexii=False, begin=0, style=None,
+                 highlight=None, cyclic=False):
     """hexdump_iter(s, width = 16, skip = True, hexii = False, begin = 0,
-                    style = {}, highlight = []) -> str generator
+                    style = None, highlight = None, cyclic = False) -> str generator
 
-    Return a hexdump-dump of a string as a generator of lines.
+    Return a hexdump-dump of a string as a generator of lines.  Unless you have
+    massive amounts of data you probably want to use :meth:`hexdump`.
 
-    Args:
-      s(str): The string to dump
-      width(int): The number of characters per line
-      skip(bool): Set to True, if repeated lines should be replaced by a "*"
-      hexii(bool): Set to True, if a hexii-dump should be returned instead of a hexdump.
-      begin(int):  Offset of the first byte to print in the left column
-      style(dict): Color scheme to use.
-      highlight(iterable): Byte values to highlight.
+    Arguments:
+        fd(file): File object to dump.  Use :meth:`StringIO.StringIO` or :meth:`hexdump` to dump a string.
+        width(int): The number of characters per line
+        skip(bool): Set to True, if repeated lines should be replaced by a "*"
+        hexii(bool): Set to True, if a hexii-dump should be returned instead of a hexdump.
+        begin(int):  Offset of the first byte to print in the left column
+        style(dict): Color scheme to use.
+        highlight(iterable): Byte values to highlight.
+        cyclic(bool): Attempt to skip consecutive, unmodified cyclic lines
 
     Returns:
-      A hexdump-dump in the form of a string.
-"""
+        A generator producing the hexdump-dump one line at a time.
+    """
     style     = style or {}
     highlight = highlight or []
 
@@ -537,10 +627,21 @@ def hexdump_iter(s, width = 16, skip = True, hexii = False, begin = 0,
             return hbyte, abyte
         cache = [style_byte(chr(b)) for b in range(256)]
 
-    for line, chunk in enumerate(lists.group(width, s)):
+    if cyclic:
+        update_cyclic_pregenerated(len(s))
+
+    numb = 0
+    while True:
+        offset = begin + numb
+        chunk = fd.read(width)
+        if chunk == '':
+            break
+        numb += len(chunk)
         # If this chunk is the same as the last unique chunk,
         # use a '*' instead.
-        if skip and (last_unique == chunk or sequential_lines(last_unique, chunk)):
+        if skip and \
+           (last_unique == chunk or \
+            (cyclic and sequential_lines(last_unique, chunk))):
             last_unique = chunk
             if not skipping:
                 yield '*'
@@ -551,8 +652,7 @@ def hexdump_iter(s, width = 16, skip = True, hexii = False, begin = 0,
         last_unique = chunk
         skipping = False
 
-        # Cenerate contents for line
-        offset    = begin+line*width
+        # Generate contents for line
         hexbytes = ''
         printable = ''
         for i, b in enumerate(chunk):
@@ -575,9 +675,53 @@ def hexdump_iter(s, width = 16, skip = True, hexii = False, begin = 0,
         line = line_fmt % {'offset': offset, 'hexbytes': hexbytes, 'printable': printable}
         yield line
 
-    line = "%08x" % (len(s) + begin)
+    line = "%08x" % (begin + numb)
     yield line
 
-def hexdump(s, width = 16, skip = True, hexii = False, begin = 0,
-            style = None, highlight = None):
-    return '\n'.join(hexdump_iter(s, width, skip, hexii, begin, style, highlight))
+def hexdump(s, width=16, skip=True, hexii=False, begin=0,
+            style=None, highlight=None, cyclic=False):
+    """hexdump(s, width = 16, skip = True, hexii = False, begin = 0,
+               style = None, highlight = None, cyclic = False) -> str generator
+
+    Return a hexdump-dump of a string.
+
+    Arguments:
+        s(str): The data to hexdump.
+        width(int): The number of characters per line
+        skip(bool): Set to True, if repeated lines should be replaced by a "*"
+        hexii(bool): Set to True, if a hexii-dump should be returned instead of a hexdump.
+        begin(int):  Offset of the first byte to print in the left column
+        style(dict): Color scheme to use.
+        highlight(iterable): Byte values to highlight.
+        cyclic(bool): Attempt to skip consecutive, unmodified cyclic lines
+
+    Returns:
+        A hexdump-dump in the form of a string.
+"""
+    s = packing.flat(s)
+    return '\n'.join(hexdump_iter(StringIO.StringIO(s),
+                                  width,
+                                  skip,
+                                  hexii,
+                                  begin,
+                                  style,
+                                  highlight,
+                                  cyclic))
+
+def negate(value, width = None):
+    """
+    Returns the two's complement of 'value'.
+    """
+    if width is None:
+        width = context.bits
+    mask = ((1<<width)-1)
+    return ((mask+1) - value) & mask
+
+def bnot(value, width=None):
+    """
+    Returns the binary inverse of 'value'.
+    """
+    if width is None:
+        width = context.bits
+    mask = ((1<<width)-1)
+    return mask ^ value
