@@ -4,6 +4,7 @@ This module includes and extends the standard module :mod:`itertools`.
 
 __all__ = [
     'bruteforce'                             ,
+    'mbruteforce'                            ,
     'chained'                                ,
     'consume'                                ,
     'cyclen'                                 ,
@@ -52,9 +53,18 @@ __all__ = [
     'tee'
 ]
 
+import collections
+import copy
+import multiprocessing
+import operator
+import random
+import time
 from itertools import *
-import collections, operator, random, copy, logging
-log = logging.getLogger(__name__)
+
+from ..context import context
+from ..log import getLogger
+
+log = getLogger(__name__)
 
 def take(n, iterable):
     """take(n, iterable) -> list
@@ -62,7 +72,7 @@ def take(n, iterable):
     Returns first `n` elements of `iterable`.  If `iterable` is a iterator it
     will be advanced.
 
-    Args:
+    Arguments:
       n(int):  Number of elements to take.
       iterable:  An iterable.
 
@@ -86,7 +96,7 @@ def take(n, iterable):
 def tabulate(func, start = 0):
     """tabulate(func, start = 0) -> iterator
 
-    Args:
+    Arguments:
       func(function):  The function to tabulate over.
       start(int):  Number to start on.
 
@@ -107,7 +117,7 @@ def consume(n, iterator):
     Advance the iterator `n` steps ahead. If `n is :const:`None`, consume
     everything.
 
-    Args:
+    Arguments:
       n(int):  Number of elements to consume.
       iterator(iterator):  An iterator.
 
@@ -138,7 +148,7 @@ def nth(n, iterable, default = None):
     Returns the element at index `n` in `iterable`.  If `iterable` is a
     iterator it will be advanced.
 
-    Args:
+    Arguments:
       n(int):  Index of the element to return.
       iterable:  An iterable.
       default(objext):  A default value.
@@ -165,14 +175,14 @@ def quantify(iterable, pred = bool):
 
     Count how many times the predicate `pred` is :const:`True`.
 
-    Args:
-      iterable:  An iterable.
-      pred:  A function that given an element from `iterable` returns either
-      :const:`True` or :const:`False`.
+    Arguments:
+        iterable:  An iterable.
+        pred:  A function that given an element from `iterable` returns either
+               ``True`` or ``False``.
 
     Returns:
       The number of elements in `iterable` for which `pred` returns
-      :const:`True`.
+      ``True``.
 
     Examples:
       >>> quantify([1, 2, 3, 4], lambda x: x % 2 == 0)
@@ -188,7 +198,7 @@ def pad(iterable, value = None):
     Pad an `iterable` with `value`, i.e. returns an iterator whoose elements are
     first the elements of `iterable` then `value` indefinitely.
 
-    Args:
+    Arguments:
       iterable:  An iterable.
       value:  The value to pad with.
 
@@ -214,7 +224,7 @@ def cyclen(n, iterable):
 
     Repeats the elements of `iterable` `n` times.
 
-    Args:
+    Arguments:
       n(int):  The number of times to repeat `iterable`.
       iterable:  An iterable.
 
@@ -235,7 +245,7 @@ def dotproduct(x, y):
 
     Computes the dot product of `x` and `y`.
 
-    Args:
+    Arguments:
       x(iterable):  An iterable.
       x(iterable):  An iterable.
 
@@ -256,7 +266,7 @@ def flatten(xss):
     returns an iterator whoose elements is the concatenation of the elements of
     `xss`.
 
-    Args:
+    Arguments:
       xss:  An iterable of iterables.
 
     Returns:
@@ -279,7 +289,7 @@ def repeat_func(func, *args, **kwargs):
     will be computed using only functions from :mod:`itertools` which are very
     fast.
 
-    Args:
+    Arguments:
       func(function):  The function to call.
       args:  Positional arguments.
       kwargs:  Keyword arguments.
@@ -318,7 +328,7 @@ def repeat_func(func, *args, **kwargs):
 def pairwise(iterable):
     """pairwise(iterable) -> iterator
 
-    Args:
+    Arguments:
       iterable:  An iterable.
 
     Returns:
@@ -342,7 +352,7 @@ def group(n, iterable, fill_value = None):
     Similar to :func:`pwnlib.util.lists.group`, but returns an iterator and uses
     :mod:`itertools` fast build-in functions.
 
-    Args:
+    Arguments:
       n(int):  The group size.
       iterable:  An iterable.
       fill_value:  The value to fill into the remaining slots of the last group
@@ -367,7 +377,7 @@ def roundrobin(*iterables):
 
     Take elements from `iterables` in a round-robin fashion.
 
-    Args:
+    Arguments:
       *iterables:  One or more iterables.
 
     Returns:
@@ -396,7 +406,7 @@ def powerset(iterable, include_empty = True):
 
     The powerset of an iterable.
 
-    Args:
+    Arguments:
       iterable:  An iterable.
       include_empty(bool):  Whether to include the empty set.
 
@@ -423,7 +433,7 @@ def unique_everseen(iterable, key = None):
     element that will be rememberes is ``key(elm)``.  Otherwise ``elm`` is
     remembered.
 
-    Args:
+    Arguments:
       iterable:  An iterable.
       key:  A function to map over each element in `iterable` before remembering
         it.  Setting to :const:`None` is equivalent to the identity function.
@@ -458,7 +468,7 @@ def unique_justseen(iterable, key = None):
     the element that will be rememberes is ``key(elm)``.  Otherwise ``elm`` is
     remembered.
 
-    Args:
+    Arguments:
       iterable:  An iterable.
       key:  A function to map over each element in `iterable` before remembering
         it.  Setting to :const:`None` is equivalent to the identity function.
@@ -482,7 +492,7 @@ def unique_window(iterable, window, key = None):
     in `iterable` the element that will be rememberes is ``key(elm)``.
     Otherwise ``elm`` is remembered.
 
-    Args:
+    Arguments:
       iterable:  An iterable.
       window(int):  The number of elements to remember.
       key:  A function to map over each element in `iterable` before remembering
@@ -520,7 +530,7 @@ def iter_except(func, exception):
     build-in :func:`iter` but uses an exception instead of a sentinel to signal
     the end.
 
-    Args:
+    Arguments:
       func:  The function to call.
       exception(exception):  The exception that signals the end.  Other
         exceptions will not be caught.
@@ -552,7 +562,7 @@ def iter_except(func, exception):
 def random_product(*args, **kwargs):
     """random_product(*args, repeat = 1) -> tuple
 
-    Args:
+    Arguments:
       args:  One or more iterables
       repeat(int):  Number of times to repeat `args`.
 
@@ -578,7 +588,7 @@ def random_product(*args, **kwargs):
 def random_permutation(iterable, r = None):
     """random_product(iterable, r = None) -> tuple
 
-    Args:
+    Arguments:
       iterable:  An iterable.
       r(int):  Size of the permutation.  If :const:`None` select all elements in
         `iterable`.
@@ -599,7 +609,7 @@ def random_permutation(iterable, r = None):
 def random_combination(iterable, r):
     """random_combination(iterable, r) -> tuple
 
-    Args:
+    Arguments:
       iterable:  An iterable.
       r(int):  Size of the combination.
 
@@ -620,7 +630,7 @@ def random_combination(iterable, r):
 def random_combination_with_replacement(iterable, r):
     """random_combination(iterable, r) -> tuple
 
-    Args:
+    Arguments:
       iterable:  An iterable.
       r(int):  Size of the combination.
 
@@ -647,7 +657,7 @@ def lookahead(n, iterable):
     Inspects the upcoming element at index `n` without advancing the iterator.
     Raises ``IndexError`` if `iterable` has too few elements.
 
-    Args:
+    Arguments:
       n(int):  Index of the element to return.
       iterable:  An iterable.
 
@@ -678,7 +688,7 @@ def lexicographic(alphabet):
     The words with symbols in `alphabet`, in lexicographic order (determined by
     the order of `alphabet`).
 
-    Args:
+    Arguments:
       alphabet:  The alphabet to draw symbols from.
 
     Returns:
@@ -698,7 +708,7 @@ def chained(func):
 
     A decorator chaining the results of `func`.  Useful for generators.
 
-    Args:
+    Arguments:
       func(function):  The function being decorated.
 
     Returns:
@@ -719,7 +729,7 @@ def chained(func):
                 yield x
     return wrapper
 
-def bruteforce(func, alphabet, length, method = 'upto', start = None):
+def bruteforce(func, alphabet, length, method = 'upto', start = None, databag = None):
     """bruteforce(func, alphabet, length, method = 'upto', start = None)
 
     Bruteforce `func` to return :const:`True`.  `func` should take a string
@@ -730,7 +740,7 @@ def bruteforce(func, alphabet, length, method = 'upto', start = None):
     The argument `start` can be used to split the search space, which is useful
     if multiple CPU cores are available.
 
-    Args:
+    Arguments:
       func(function):  The function to bruteforce.
       alphabet:  The alphabet to draw symbols from.
       length:  Longest string to try.
@@ -738,7 +748,7 @@ def bruteforce(func, alphabet, length, method = 'upto', start = None):
         try strings of length ``length`` and if 'downfrom' try strings of length
         ``length .. 1``.
       start: a tuple ``(i, N)`` which splits the search space up into `N` pieces
-        d starts at piece `i`.  :const:`None` is equivalent to ``(0, 1)``.
+        and starts at piece `i` (1..N). :const:`None` is equivalent to ``(1, 1)``.
 
     Returns:
       A string `s` such that ``func(s)`` returns :const:`True` or :const:`None`
@@ -770,7 +780,7 @@ def bruteforce(func, alphabet, length, method = 'upto', start = None):
     if method == 'fixed':
         total_iterations = len(alphabet) ** length
     else:
-        total_iterations = len(alphabet) ** (length + 1) - 2
+        total_iterations = (len(alphabet) ** (length + 1) / (len(alphabet) - 1)) - 1
 
     if start is not None:
         i, N = start
@@ -793,21 +803,112 @@ def bruteforce(func, alphabet, length, method = 'upto', start = None):
         if rest >= i:
             chunk_size += 1
 
-        consume(iterator, starting_point)
-        iterator = take(chunk_size, iterator)
         total_iterations = chunk_size
 
     h = log.waitfor('Bruteforcing')
     cur_iteration = 0
+    if start != None:
+        consume(i, iterator)
     for e in iterator:
         cur = ''.join(e)
         cur_iteration += 1
         if cur_iteration % 2000 == 0:
             progress = 100.0 * cur_iteration / total_iterations
             h.status('Trying "%s", %0.3f%%' % (cur, progress))
+            if databag:
+                databag["current_item"] = cur
+                databag["items_done"] = cur_iteration
+                databag["items_total"] = total_iterations
         res = func(cur)
         if res:
             h.success('Found key: "%s"' % cur)
             return cur
+        if start != None:
+            consume(N - 1, iterator)
 
     h.failure('No matches found')
+
+
+
+def mbruteforce(func, alphabet, length, method = 'upto', start = None, threads = None):
+    """mbruteforce(func, alphabet, length, method = 'upto', start = None, threads = None)
+
+    Same functionality as bruteforce(), but multithreaded.
+
+    Arguments:
+      func, alphabet, length, method, start: same as for bruteforce()
+      threads: Amount of threads to spawn, default is the amount of cores.
+    """
+
+    def bruteforcewrap(func, alphabet, length, method, start, databag):
+        oldloglevel = context.log_level
+        context.log_level = 'critical'
+        res = bruteforce(func, alphabet, length, method=method, start=start, databag=databag)
+        context.log_level = oldloglevel
+        databag["result"] = res
+
+    if start == None:
+        start = (1, 1)
+
+    if threads == None:
+        try:
+            threads = multiprocessing.cpu_count()
+        except NotImplementedError:
+            threads = 1
+
+    h = log.waitfor('MBruteforcing')
+    processes = [None] * threads
+    shareddata = [None] * threads
+
+    (i2, N2) = start
+    totalchunks = threads * N2
+
+    for i in range(threads):
+        shareddata[i] = multiprocessing.Manager().dict()
+        shareddata[i]['result'] = None
+        shareddata[i]['current_item'] = ""
+        shareddata[i]['items_done'] = 0
+        shareddata[i]['items_total'] = 0
+
+        chunkid = (i2-1) + (i * N2) + 1
+
+        processes[i] = multiprocessing.Process(target=bruteforcewrap,
+                args=(func, alphabet, length, method, (chunkid, totalchunks),
+                        shareddata[i]))
+        processes[i].start()
+
+    done = False
+
+    while not done:
+        # log status
+        current_item_list = ",".join(["\"%s\"" % x["current_item"]
+                                for x in shareddata if x != None])
+        items_done = sum([x["items_done"] for x in shareddata if x != None])
+        items_total = sum([x["items_total"] for x in shareddata if x != None])
+
+        progress = 100.0 * items_done / items_total if items_total != 0 else 0.0
+
+        h.status('Trying %s -- %0.3f%%' % (current_item_list, progress))
+
+        # handle finished threads
+        for i in range(threads):
+            if processes[i] and processes[i].exitcode != None:
+                # thread has terminated
+                res = shareddata[i]["result"]
+                processes[i].join()
+                processes[i] = None
+
+                # if successful, kill all other threads and return success
+                if res != None:
+                    for i in range(threads):
+                        if processes[i] != None:
+                            processes[i].terminate()
+                            processes[i].join()
+                            processes[i] = None
+                    h.success('Found key: "%s"' % res)
+                    return res
+
+                if all([x == None for x in processes]):
+                    done = True
+        time.sleep(0.3)
+    h.failure('No matches found')
