@@ -1,5 +1,9 @@
-import time, errno, logging
+import errno
+import socket
+import time
+
 from .. import tubes
+from ..log import getLogger
 
 try:
     import psutil
@@ -7,7 +11,7 @@ try:
 except ImportError:
     _ok_import = False
 
-log = logging.getLogger(__name__)
+log = getLogger(__name__)
 
 if _ok_import:
     all_pids = psutil.pids
@@ -17,29 +21,40 @@ def pidof(target):
 
     Get PID(s) of `target`.  The returned PID(s) depends on the type of `target`:
 
-        - :class:`str`: PIDs of all processes with a name matching `target`.
+    - :class:`str`: PIDs of all processes with a name matching `target`.
+    - :class:`pwnlib.tubes.process.process`: singleton list of the PID of `target`.
+    - :class:`pwnlib.tubes.sock.sock`: singleton list of the PID at the
+      remote end of `target` if it is running on the host.  Otherwise an
+      empty list.
 
-        - :class:`pwnlib.tubes.process.process`: singleton list of the PID of `target`.
-
-        - :class:`pwnlib.tubes.sock.sock`: singleton list of the PID at the
-        remote end of `target` if it is running on the host.  Otherwise an
-        empty list.
-
-    Args:
+    Arguments:
         target(object):  The target whose PID(s) to find.
 
     Returns:
         A list of found PIDs.
     """
-    if isinstance(target, tubes.sock.sock):
+    if isinstance(target, tubes.ssh.ssh_channel):
+        return [target.pid]
+
+    elif isinstance(target, tubes.sock.sock):
          local  = target.sock.getsockname()
          remote = target.sock.getpeername()
 
-         def match(p):
+         def match(c):
              return (c.raddr, c.laddr, c.status) == (local, remote, 'ESTABLISHED')
 
          return [c.pid for c in psutil.net_connections() if match(c)]
 
+    elif isinstance(target, tuple):
+        host, port = target
+
+        host = socket.gethostbyname(host)
+
+        def match(c):
+            return c.raddr == (host, port)
+
+        return [c.pid for c in psutil.net_connections() if match(c)]
+
     elif isinstance(target, tubes.process.process):
          return [target.proc.pid]
 
@@ -49,7 +64,7 @@ def pidof(target):
 def pid_by_name(name):
     """pid_by_name(name) -> int list
 
-    Args:
+    Arguments:
         name (str): Name of program.
 
     Returns:
@@ -60,28 +75,35 @@ def pid_by_name(name):
         True
     """
     def match(p):
-         if p.name() == name:
-             return True
-         try:
-             if p.exe() == name:
-                 return True
-         except:
-             pass
-         return False
+        if p.status() == 'zombie':
+            return False
+        if p.name() == name:
+            return True
+        try:
+            if p.exe() == name:
+                return True
+        except Exception:
+            pass
+        return False
 
-    return [p.pid for p in psutil.process_iter() if match(p)]
+    processes = (p for p in psutil.process_iter() if match(p))
+
+    processes = sorted(processes, key=lambda p: p.create_time())
+
+    return list(reversed([p.pid for p in processes]))
 
 def name(pid):
     """name(pid) -> str
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
         Name of process as listed in ``/proc/<pid>/status``.
 
     Example:
-        >>> name(os.getpid()) == os.path.basename(sys.argv[0])
+        >>> pid = pidof('init')[0]
+        >>> name(pid) == 'init'
         True
     """
     return psutil.Process(pid).name()
@@ -89,7 +111,7 @@ def name(pid):
 def parent(pid):
     """parent(pid) -> int
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -98,13 +120,13 @@ def parent(pid):
     """
     try:
          return psutil.Process(pid).parent().pid
-    except:
+    except Exception:
          return 0
 
 def children(ppid):
     """children(ppid) -> int list
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -115,7 +137,7 @@ def children(ppid):
 def ancestors(pid):
     """ancestors(pid) -> int list
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -130,7 +152,7 @@ def ancestors(pid):
 def descendants(pid):
     """descendants(pid) -> dict
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -152,7 +174,7 @@ def descendants(pid):
 def exe(pid):
     """exe(pid) -> str
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -163,7 +185,7 @@ def exe(pid):
 def cwd(pid):
     """cwd(pid) -> str
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -175,7 +197,7 @@ def cwd(pid):
 def cmdline(pid):
     """cmdline(pid) -> str list
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -186,7 +208,7 @@ def cmdline(pid):
 def stat(pid):
     """stat(pid) -> str list
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -203,7 +225,7 @@ def stat(pid):
 def starttime(pid):
     """starttime(pid) -> float
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -216,7 +238,7 @@ def status(pid):
 
     Get the status of a process.
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -240,7 +262,7 @@ def status(pid):
 def tracer(pid):
     """tracer(pid) -> int
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -256,7 +278,7 @@ def tracer(pid):
 def state(pid):
     """state(pid) -> str
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
@@ -273,7 +295,7 @@ def wait_for_debugger(pid):
 
     Sleeps until the process with PID `pid` is being traced.
 
-    Args:
+    Arguments:
         pid (int): PID of the process.
 
     Returns:
